@@ -1,9 +1,3 @@
-generation_time <- get_generation_time(disease = "SARS-CoV-2", source = "ganyani", max_value = 5)
-reporting_delay <- dist_spec(
-  mean = log(3), mean_sd = 0.1,
-  sd = log(2), sd_sd = 0.1, max = 5
-)
-
 futile.logger::flog.threshold("FATAL")
 # uses example case vector
 cases <- EpiNow2::example_confirmed[1:30]
@@ -17,9 +11,9 @@ df_non_zero <- function(df) {
 }
 
 out <- suppressWarnings(regional_epinow(
-  reported_cases = cases,
-  generation_time = generation_time_opts(generation_time),
-  delays = delay_opts(reporting_delay),
+  data = cases,
+  generation_time = generation_time_opts(example_generation_time),
+  delays = delay_opts(example_reporting_delay),
   stan = stan_opts(
     samples = 25, warmup = 25,
     chains = 2,
